         ---------------------------
         -- Mighty Mouse Routines --   "Here I come to save the day!"
         ---------------------------
-- You need DOS mouse support for these routines.

-- modified May 24, 1998 by Pete Eberlein <xseal@harborside.com>
--   get_mouse() returns mouse events as usual
--   the first call to mouse_pointer activates the extended mouse
-- added:
--   mouse_cursor(map, mask)
--     mask is AND'ed to screen, then map is XOR'ed on top
--   mouse_inside(x1, y1, x2, y2)
--     returns true if the mouse is in the bounding box (x1,y1)-(x2,y2)
--   poll_mouse()
--     returns {buttons, x_pos, y_pos} of the mouse using get_mouse

include machine.e
include image.e

-- Mouse Event Numbers:
global constant MOVE = 1,       -- track every movement of the mouse
                LEFT_DOWN = 2,  -- the rest are button pressing/releasing
                LEFT_UP = 4,
                RIGHT_DOWN = 8,
                RIGHT_UP = 16,
                MIDDLE_DOWN = 32,
                MIDDLE_UP = 64

constant M_GET_MOUSE = 14,
         M_MOUSE_EVENTS = 15
--constant         M_MOUSE_POINTER = 24

integer extended_mouse
    extended_mouse = 0

sequence m_back, m_pos, m_rawpos, m_bord, m_map, m_mask
integer m_visible, m_buts, m_events, m_moved
integer mWide, mTall

global function poll_mouse()
    sequence REGS, temp
    integer dy, dx, bx1, bx2, bx3, bx4, tx1, tx2

    REGS = repeat(0, 10)
    REGS[REG_AX] = #B              -- get relative motion counters
    REGS = dos_interrupt(#33, REGS)
    temp = REGS[REG_CX] & REGS[REG_DX]
    m_rawpos = m_rawpos + temp - 65536 * (temp > 32767)
    m_rawpos = m_rawpos * (m_rawpos >= 0) * (m_rawpos <= m_bord)
             + m_bord * (m_rawpos > m_bord)
        
    REGS[REG_AX] = 3               -- get mouse buttons in BX
    REGS = dos_interrupt(#33, REGS)
                
    m_moved = compare(m_rawpos, m_pos)
    if m_moved then
        if m_visible = 0 then
            temp = save_image(m_rawpos, m_rawpos+{mWide,mTall}-1)
            dx = m_rawpos[1] - m_pos[1]
            dy = m_rawpos[2] - m_pos[2]
            if (dy > -mTall and dy < mTall) 
            and (dx > -mWide and dx < mWide) then
                if dx >= 0 and dx < mWide then   -- mouse moved to the right
                    tx1 = 1           tx2 = mWide - dx
                    bx1 = 1 + dx      bx2 = mWide
                    bx3 = 1           bx4 = dx
                    dx = 0
                elsif dx > -mWide and dx < 0 then -- mouse moved to the left
                    tx1 = 1 - dx      tx2 = mWide
                    bx1 = 1           bx2 = mWide + dx
                    bx3 = mWide+1+dx  bx4 = mWide
                    dx = mWide + dx
                end if
                dy = - dy
                for y = 1 to length(m_back) do
                    dy = dy + 1
                    if dy >= 1 and dy <= mTall then
                        temp[dy][tx1..tx2] = m_back[y][bx1..bx2]
                        pixel(m_back[y][bx3..bx4], m_pos+{dx,y-1})
                    else
                        pixel(m_back[y], m_pos+{0,y-1})
                    end if
                end for
            else  -- new mouse image doesn't overlap old background
                display_image(m_pos, m_back)
            end if
            display_image(m_rawpos, or_bits(and_bits(temp, m_mask), m_map))
            m_back = temp
        end if
        m_pos = m_rawpos
    end if
    return REGS[REG_BX] & m_pos
end function

global function get_mouse()
-- report mouse events,
-- returns -1 if no mouse event,
-- otherwise returns {event#, x-coord, y-coord}
    object me -- mouse events
    integer temp
    
    if extended_mouse then
        me = poll_mouse()
        temp = 0
        if me[1] != m_buts then
            temp = xor_bits(me[1], m_buts)
            temp = 2*(and_bits(temp,1) * (1+and_bits(m_buts,1))  --LEFT
                    + and_bits(temp,2) * (2+and_bits(m_buts,2))  --RIGHT
                    + and_bits(temp,4) * (4+and_bits(m_buts,4))) --MIDDLE
            m_buts = me[1]
        end if
        me[1] = and_bits(temp + m_moved, m_events)
        if me[1] = 0 then
            me = -1
        end if
    else -- not extended_mouse
        me = machine_func(M_GET_MOUSE, 0)
    end if
    return me
end function

global procedure mouse_events(integer events)
-- select the mouse events to be reported by get_mouse()
-- e.g. mouse_events(LEFT_UP + LEFT_DOWN + RIGHT_DOWN)
    m_events = events
    if not extended_mouse then
        machine_proc(M_MOUSE_EVENTS, events)
    end if
end procedure


global procedure mouse_pointer(integer show_it)
-- show (1) or hide (0) the mouse pointer
    sequence vparams, regs
    if not extended_mouse then
        vparams = video_config()
        
        m_bord = {vparams[VC_XPIXELS], vparams[VC_YPIXELS]} - 1

        regs = repeat(0, 10)
        regs = dos_interrupt(#33, regs) -- reset mouse

        extended_mouse = 1
    
        m_pos = {0,0}
        m_rawpos = {0,0}
        m_back = {{}}
        m_events = MOVE + LEFT_DOWN + LEFT_UP 
                        + RIGHT_DOWN + RIGHT_UP 
                        + MIDDLE_DOWN + MIDDLE_UP
        m_visible = 1  -- not visible
        m_buts = 0     -- no buttons pressed

        m_map = {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                { 0,15, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                { 0,15,15, 0, 0, 0, 0, 0, 0, 0, 0},
                { 0,15,15,15, 0, 0, 0, 0, 0, 0, 0},
                { 0,15,15,15,15, 0, 0, 0, 0, 0, 0},
                { 0,15,15,15,15,15, 0, 0, 0, 0, 0},
                { 0,15,15,15,15,15,15, 0, 0, 0, 0},
                { 0,15,15,15,15,15,15,15, 0, 0, 0},
                { 0,15,15,15,15,15,15,15,15, 0, 0},
                { 0,15,15,15,15,15, 0, 0, 0, 0, 0},
                { 0,15,15, 0,15,15, 0, 0, 0, 0, 0},
                { 0,15, 0, 0, 0,15,15, 0, 0, 0, 0},
                { 0, 0, 0, 0, 0,15,15, 0, 0, 0, 0},
                { 0, 0, 0, 0, 0, 0,15,15, 0, 0, 0},
                { 0, 0, 0, 0, 0, 0,15,15, 0, 0, 0},
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}
        m_mask = #FF * {
                { 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                { 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1},
                { 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1},
                { 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1},
                { 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
                { 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
                { 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                { 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
                { 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
                { 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1},
                { 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1},
                { 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1},
                { 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1}}
        mWide = length(m_map[1])
        mTall = length(m_map)
    end if
    if show_it then
        if m_visible > 0 then
            m_visible = m_visible - 1
        end if
        if m_visible = 0 then
            m_back = save_image(m_pos, m_pos+{mWide,mTall}-1)
            display_image(m_pos, xor_bits(and_bits(m_back, m_mask),
m_map))
        end if
    else
        if m_visible = 0 then
            display_image(m_pos, m_back)
        end if
        m_visible = m_visible + 1
--    else
--        machine_proc(M_MOUSE_POINTER, show_it)
    end if
end procedure


global procedure mouse_cursor(sequence map, sequence mask)
    mouse_pointer(0)
    m_map = map
    m_mask = #FF * (mask != 0)
    mWide = length(m_map[1])
    mTall = length(m_map)
    mouse_pointer(1)
end procedure

global function mouse_inside(integer x1, integer y1, integer x2, integer y2)
    return m_pos[1] >= x1
       and m_pos[2] >= y1
       and m_pos[1] <= x2
       and m_pos[2] <= y2
end function

